import gleam/dynamic/decode.{type Decoder}
import gleam/json
import models/account.{type Account}
import models/agent.{type Agent}
import models/agent_symbol.{type AgentSymbol}
import models/contract.{type Contract}
import models/faction.{type Faction}
import models/faction_symbol.{type FactionSymbol}
import models/ship.{type Ship}
import utils/auth.{type AccountToken, type AgentToken, AccountAuth, AgentAuth}
import utils/request as request_utils
import utils/response as response_utils
import utils/types.{type ApiResponse}

pub type GetAccountResponse {
  GetAccountResponse(account: Account)
}

fn get_account_response_decoder() -> Decoder(GetAccountResponse) {
  use account <- decode.field("account", account.decoder())
  decode.success(GetAccountResponse(account:))
}

pub fn get_account(token: AgentToken) -> ApiResponse(GetAccountResponse) {
  let request = request_utils.get(AgentAuth(token), "/my/account")
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(response.body, get_account_response_decoder())
    _ -> response_utils.error_parser(response.body)
  }
}

pub type RegisterNewAgentResponse {
  RegisterNewAgentResponse(
    token: AgentToken,
    agent: Agent,
    faction: Faction,
    contract: Contract,
    ships: List(Ship),
  )
}

fn register_new_agent_response_decoder() -> Decoder(RegisterNewAgentResponse) {
  use token <- decode.field("token", auth.agent_token_decoder())
  use agent <- decode.field("agent", agent.decoder())
  use faction <- decode.field("faction", faction.decoder())
  use contract <- decode.field("contract", contract.decoder())
  use ships <- decode.field("ships", decode.list(ship.decoder()))
  decode.success(RegisterNewAgentResponse(
    token:,
    agent:,
    faction:,
    contract:,
    ships:,
  ))
}

pub fn register_new_agent(
  token: AccountToken,
  agent_symbol: AgentSymbol,
  faction_symbol: FactionSymbol,
) -> ApiResponse(RegisterNewAgentResponse) {
  let request =
    request_utils.post_json(
      AccountAuth(token),
      "/register",
      json.object([
        #("symbol", json.string(agent_symbol)),
        #("faction", json.string(faction_symbol.to_string(faction_symbol))),
      ]),
    )
  use response <- request_utils.try_send(request)
  case response.status {
    201 ->
      response_utils.data_parser(
        response.body,
        register_new_agent_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}
