import gleam/dynamic/decode.{type Decoder}
import models/agent.{type Agent}
import models/agent_event.{type AgentEvent}
import models/public_agent.{type PublicAgent}
import utils/auth.{type AgentToken, AgentAuth}
import utils/request as request_utils
import utils/response as response_utils
import utils/types.{type ApiResponse, type PagedData}

pub type ListPublicAgentsResponse {
  ListPublicAgentsResponse(agents: List(PublicAgent))
}

fn list_public_agents_response_decoder() -> Decoder(ListPublicAgentsResponse) {
  use agents <- decode.then(decode.list(public_agent.decoder()))
  decode.success(ListPublicAgentsResponse(agents:))
}

pub fn list_public_agents(
  token: AgentToken,
  page: Int,
  limit: Int,
) -> ApiResponse(PagedData(ListPublicAgentsResponse)) {
  let request = request_utils.get_page(AgentAuth(token), "/agents", page, limit)
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.paged_data_parser(
        response.body,
        list_public_agents_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}

pub type GetPublicAgentResponse {
  GetPublicAgentResponse(agent: PublicAgent)
}

fn get_public_agent_response_decoder() -> Decoder(GetPublicAgentResponse) {
  use agent <- decode.then(public_agent.decoder())
  decode.success(GetPublicAgentResponse(agent:))
}

pub fn get_public_agent(
  token: AgentToken,
  agent_symbol: String,
) -> ApiResponse(GetPublicAgentResponse) {
  let request = request_utils.get(AgentAuth(token), "/agents/" <> agent_symbol)
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(
        response.body,
        get_public_agent_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}

pub type GetAgentResponse {
  GetAgentResponse(agent: Agent)
}

fn get_agent_response_decoder() -> decode.Decoder(GetAgentResponse) {
  use agent <- decode.then(agent.decoder())
  decode.success(GetAgentResponse(agent:))
}

pub fn get_agent(token: AgentToken) -> ApiResponse(GetAgentResponse) {
  let request = request_utils.get(AgentAuth(token), "/my/account")
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(response.body, get_agent_response_decoder())
    _ -> response_utils.error_parser(response.body)
  }
}

pub type GetAgentEventsResponse {
  GetAgentEventsResponse(events: List(AgentEvent))
}

fn get_agent_events_response_decoder() -> Decoder(GetAgentEventsResponse) {
  use events <- decode.then(decode.list(agent_event.decoder()))
  decode.success(GetAgentEventsResponse(events:))
}

pub fn get_agent_events(
  token: AgentToken,
) -> ApiResponse(GetAgentEventsResponse) {
  let request = request_utils.get(AgentAuth(token), "/my/agent/events")
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(
        response.body,
        get_agent_events_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}
