import gleam/dynamic/decode.{type Decoder}
import gleam/json
import models/agent.{type Agent}
import models/contract.{type Contract}
import models/ship_cargo.{type ShipCargo}
import models/ship_symbol.{type ShipSymbol}
import models/trade_symbol.{type TradeSymbol}
import utils/auth.{type AgentToken, AgentAuth}
import utils/request as request_utils
import utils/response as response_utils
import utils/types.{type ApiResponse, type PagedData}

pub type ListContractsResponse {
  ListContractsResponse(contracts: List(Contract))
}

fn list_contracts_response_decoder() -> Decoder(ListContractsResponse) {
  use contracts <- decode.then(decode.list(contract.decoder()))
  decode.success(ListContractsResponse(contracts:))
}

pub fn list_contracts(
  token: AgentToken,
  page: Int,
  limit: Int,
) -> ApiResponse(PagedData(ListContractsResponse)) {
  let request =
    request_utils.get_page(AgentAuth(token), "/my/contracts", page, limit)
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.paged_data_parser(
        response.body,
        list_contracts_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}

pub type GetContractResponse {
  GetContractResponse(contract: Contract)
}

fn get_contract_response_decoder() -> Decoder(GetContractResponse) {
  use contract <- decode.then(contract.decoder())
  decode.success(GetContractResponse(contract:))
}

pub fn get_contract(
  token: AgentToken,
  contract_id: String,
) -> ApiResponse(GetContractResponse) {
  let request =
    request_utils.get(AgentAuth(token), "/my/contracts/" <> contract_id)
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(response.body, get_contract_response_decoder())
    _ -> response_utils.error_parser(response.body)
  }
}

pub type AcceptContractResponse {
  AcceptContractResponse(contract: Contract, agent: Agent)
}

fn accept_contract_response_decoder() -> Decoder(AcceptContractResponse) {
  use contract <- decode.field("contract", contract.decoder())
  use agent <- decode.field("agent", agent.decoder())
  decode.success(AcceptContractResponse(contract:, agent:))
}

pub fn accept_contract(
  token: AgentToken,
  contract_id: String,
) -> ApiResponse(AcceptContractResponse) {
  let request =
    request_utils.post(
      AgentAuth(token),
      "/my/contracts/" <> contract_id <> "/accept",
    )
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(
        response.body,
        accept_contract_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}

pub type FulfillContractResponse {
  FulfillContractResponse(contract: Contract, agent: Agent)
}

fn fulfill_contract_response_decoder() -> Decoder(FulfillContractResponse) {
  use contract <- decode.field("contract", contract.decoder())
  use agent <- decode.field("agent", agent.decoder())
  decode.success(FulfillContractResponse(contract:, agent:))
}

pub fn fulfill_contract(
  token: AgentToken,
  contract_id,
) -> ApiResponse(FulfillContractResponse) {
  let request =
    request_utils.post(
      AgentAuth(token),
      "/my/contracts/" <> contract_id <> "/fulfill",
    )
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(
        response.body,
        fulfill_contract_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}

pub type DeliverContractCargoResponse {
  DeliverContractCargoResponse(contract: Contract, cargo: ShipCargo)
}

fn deliver_contract_cargo_response_decoder() -> Decoder(
  DeliverContractCargoResponse,
) {
  use contract <- decode.field("contract", contract.decoder())
  use cargo <- decode.field("cargo", ship_cargo.decoder())
  decode.success(DeliverContractCargoResponse(contract:, cargo:))
}

pub fn deliver_contract_cargo(
  token: AgentToken,
  contract_id: String,
  ship_symbol: ShipSymbol,
  trade_symbol: TradeSymbol,
  units: Int,
) -> ApiResponse(DeliverContractCargoResponse) {
  let request =
    request_utils.post_json(
      AgentAuth(token),
      "/my/contracts/" <> contract_id <> "/deliver",
      json.object([
        #("shipSymbol", json.string(ship_symbol)),
        #("tradeSymbol", json.string(trade_symbol.to_string(trade_symbol))),
        #("units", json.int(units)),
      ]),
    )
  use response <- request_utils.try_send(request)
  case response.status {
    200 ->
      response_utils.data_parser(
        response.body,
        deliver_contract_cargo_response_decoder(),
      )
    _ -> response_utils.error_parser(response.body)
  }
}
